#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::HTTP;
use Wanha::HTML;
use Wanha::DB;
use Wanha::Session;
use Wanha::Link;
use Wanha::URL;
use Wanha::URLList;
use Wanha::Common;


=item get_latest
Get latest URLs

0: latest url id
1: latest seen url id
2: count
3: date format
@: ignored chans
r: array of hash (url_id, url_url, url_count, url_date, chan_id, chan_name,
		nick_id, nick_name)
=cut
sub get_latest($$$$@) {
	my $latest_id = shift;
	my $latest_seen = shift;
	my $count = shift;
	my $date_fmt = shift;
	my @ignored = @_;

	my $ignore;
	my ($query, $result);
	my @ret;

	$count = &Wanha::Common::min($count, 100);

	my @prep_ids = split(//, "abcdefghijklmnopqrstuvwxyz" .
			"ABCDEFGHIJKLMNOPQRSTUVWXYZ");
	$ignore = "";
	if ($#ignored >= 0) {
		for (my $i = 0; $i <= $#ignored; $i++) {
			$ignore .= sprintf(" AND urls.chan != &d%s&",
					$prep_ids[$i]);
		}
		$ignore =~ s/ AND (.*)/AND ($1) /;
		$ignore = &Wanha::DB::prepare($ignore);
		for (my $i = 0; $i < $#ignored; $i++) {
			$ignore = &Wanha::DB::set($ignore,
					$prep_ids[$i], $ignored[$i]);
		}
	}

	my $to_char = &Wanha::DB::get_date_fmt("urls.time", "time", $date_fmt);
	$query = &Wanha::DB::prepare(
			"SELECT urls.id, urls.url, urls.count, $to_char, " .
				"chans.id, chans.chan, " .
				"nicks.id, nicks.nick " .
			"FROM urls, chans, nicks " .
			"WHERE urls.id > &da& AND urls.id <= &db& $ignore " .
				"AND urls.nick = nicks.id " .
				"AND urls.chan = chans.id " .
			"ORDER BY urls.id ASC " .
			"LIMIT &dc&");
	$query = &Wanha::DB::set($query,
			"a", $latest_seen,
			"b", $latest_id,
			"c", $count);
	$result = &Wanha::DB::query($query);
	(&Wanha::DB::row_count($result) == 0) && return ();

	while (my ($url_id, $url_url, $url_count, $url_date,
				$chan_id, $chan_name,
				$nick_id, $nick_name) =
			&Wanha::DB::get_row($result)) {
		my %t = (
			'url_id' => $url_id,
			'url_url' => $url_url,
			'url_date' => $url_date,
			'url_count' => $url_count,
			'chan_id' => $chan_id,
			'chan_name' => $chan_name,
			'nick_id' => $nick_id,
			'nick_name' => $nick_name
			);
		push(@ret, \%t);
	}
	return @ret;
}


=item latest
Print latest URLs

0: sid
=cut
sub latest($) {
	my ($sid) = @_;

	my %data = &Wanha::Cookie::get_info($sid);
	my $latest_id = &Wanha::URL::get_latest_id();
	my @ignore = &Wanha::Cookie::get_ignore_chans($sid);
	my @urls = &get_latest($latest_id, $data{'latest'}, $data{'urls'},
			$data{'latest_date'}, @ignore);
	if ($#urls >= 0) {
		my %latest = %{$urls[$#urls]};
		$latest_id = $latest{'url_id'};
	}

	if ($data{'reset'} == 1 && $data{'latest'} < $latest_id) {
		&Wanha::Cookie::set_latest($sid, $latest_id);
	}

	if ($#urls == -1) {
		&Wanha::Lang::printf("<h1><&latest-no-new-urls&></h1>\n");
		return;
	}

	&Wanha::Lang::printf("<h1><&latest-label&></h1>\n");
	&Wanha::URLList::print(\@urls, "url", "nick", "chan", "date", "count");

	if ($data{'reset'} == 0) {
		&Wanha::Lang::printf(
				"<p><a href=\"%s\"><&latest-reset&></a></p>\n",
				&Wanha::Link::latest(1, $latest_id));
	}
}


=item main
Main function
=cut
sub main() {
	my $sid;
	my $phase;

	# Updates cookie automagically
	$sid = &Wanha::Session::get_id();
	&Wanha::Session::check_sid(
			$sid, &Wanha::Link::session(1, &Wanha::Link::latest()));

	if (&Wanha::HTTP::get("reset", 0) == 1) {
		my $latest = &Wanha::HTTP::get_numeric("latest", -1);
		$latest = &Wanha::URL::get_latest_id() if ($latest == -1);
		&Wanha::Cookie::set_latest($sid, $latest);
		&Wanha::HTTP::redirect(&Wanha::Link::latest());
		&Wanha::HTTP::print_header();
		exit(0);
	}

	my @extra;
	my $line = sprintf("<link rel=\"alternate\" " .
			"href=\"%s\" " .
			"type=\"application/atom+xml\" " .
			"title=\"ATOM - wanha\" />",
			&Wanha::Link::atom_chan());
	push(@extra, $line);

	&Wanha::HTTP::print_header();
	&Wanha::HTML::print_header(undef, @extra);

	&Wanha::Session::print_links();

	&latest($sid);

	&Wanha::HTML::print_footer(&Wanha::Common::process_time());
}


&main();
